from urllib.parse import urlencode,quote
import requests, collections

def process_resp(rows):
	header = rows.pop(0)
	rowc = collections.namedtuple("Row",header)
	ret = []
	e = []
	for row in rows:
		if len(row)==len(header): # data row
			ret.append(rowc._make(row))
		elif len(row)==1: # continue key
			e.append(row)
		# leave off empty separator row
	return ret+e

def get(url,postproc=process_resp,**kwargs):
	kwargs["url"]=url
	kwargs["output"]="json" # force JSON output
	for k in kwargs.keys():
		if type(kwargs[k])==list:
			kwargs[k]=",".join(kwargs[k])
	r = requests.get("http://web.archive.org/cdx/search/cdx?"+urlencode(kwargs,safe="",quote_via=quote))
	r.raise_for_status()
	return postproc(r.json())



from archivecdx.grab import get
from time import sleep

# no resumption key/counter support, if you need them then make a PR and handle them yourself
DONT_USE = "showResumeKey resumeKey showDupeCount showSkipCount".split()

class Listing:
	def __init__(self,url,delay=1,**kwargs):
		self.url = url
		self.delay = delay
		self.kwargs = kwargs
		for key in kwargs:
			if key.endswith("_"):
				self.kwargs[key[:-1]]=self.kwargs[key]
				del self.kwargs[key]
		self.pre_process()
		for key in DONT_USE:
			if key in self.kwargs: del self.kwargs[key]
		self.listing = get(self.url,**self.kwargs)
	def pre_process(self):
		return
	def __getitem__(self,k):
		return self.listing[k]

class PaginatedListing(Listing):
	def pre_process(self):
		if not "page_count" in self.kwargs:
			self.kwargs["showNumPages"]="true"
			self.page_count = get(self.url,lambda x: x,**self.kwargs)
			del self.kwargs["showNumPages"]
			sleep(self.delay)
		else:
			self.page_count = self.kwargs["page_count"]
			del self.kwargs["page_count"]
		if "page" not in self.kwargs: self.kwargs["page"]=0
		self.kwargs["page"] = self.page_count if self.kwargs["page"]>self.page_count else self.kwargs["page"]
		self.page = self.kwargs["page"]
	@property
	def next_page(self):
		if self.page>self.page_count:
			return None # no next page
		self.kwargs["page"]=self.page+1
		self.kwargs["page_count"]=self.page_count
		return self.__class__(self.url,**self.kwargs)



from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='archive-cdx',  # Required

    version='0.2.0',  # Required

    description='A library to interact with the archive.org Wayback Machine CDX server.',  # Optional

    long_description=long_description,  # Optional

    # This should be a valid link to your project's main homepage.
    #
    # This field corresponds to the "Home-Page" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#home-page-optional
    url='https://github.com/MineRobber9000/archive-cdx',  # Optional

    # This should be your name or the name of the organization which owns the
    # project.
    author='Robert "khuxkm" Miles',  # Optional

    # This should be a valid email address corresponding to the author listed
    # above.
    author_email='khuxkm@tilde.ninja',  # Optional

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    # This field adds keywords for your project which will appear on the
    # project page. What does your project relate to?
    #
    # Note that this is a string of words separated by whitespace, not a list.
    keywords='wayback-machine cdx',  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests'],  # Optional

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    #extras_require={  # Optional
    #    'dev': ['check-manifest'],
    #    'test': ['coverage'],
    #},

    # If there are data files included in your packages that need to be
    # installed, specify them here.
    #
    # If using Python 2.6 or earlier, then these have to be included in
    # MANIFEST.in as well.
    #package_data={  # Optional
    #    'sample': ['package_data.dat'],
    #},

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    #data_files=[('my_data', ['data/data_file'])],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    #entry_points={  # Optional
    #    'console_scripts': [
    #        'sample=sample:main',
    #    ],
    #},

    # List additional URLs that are relevant to your project as a dict.
    #
    # This field corresponds to the "Project-URL" metadata fields:
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    #
    # Examples listed include a pattern for specifying where the package tracks
    # issues, where the source is hosted, where to say thanks to the package
    # maintainers, and where to support the project financially. The key is
    # what's used to render the link text on PyPI.
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/MineRobber9000/archive-cdx/issues',
        'Source': 'https://github.com/MineRobber9000/archive-cdx/',
    },
)
